require 'entities.ship'

local function add_explosion(x,y)
	local explosion = {}
	local speed = 0.05
	local anim = newAnimation(IMAGES['explosion.png'], 32, 45, speed, 16)
	anim:setMode('once')

	local ox,oy = 16,22
	local angle,rot = 0, math.random(0,math.pi)
	function explosion:draw()
		love.graphics.setColor(255,255,255)
		anim:draw(x,y, angle, 2,2, ox,oy)
	end

	function explosion:update(dt)
		angle = angle + rot * dt
		anim:update(dt)
	end

	Timer.add(speed * 16, function() ENTITIES[explosion] = nil end)

	ENTITIES[explosion] = explosion
end

local function drop(sprite, what, x,y)
	local self = {}
	self.shape = HC.addCircle(x,y,16)
	self.shape.class = what
	self.shape.self = self

	function self.draw()
		love.graphics.setColor(255,255,255)
		love.graphics.draw(sprite, x-16,y-16)
	end
	function self.update() end

	ENTITIES[self] = self
end

local function spawnShip(pos, player)
	local ship = Ship( pos.x,pos.y, 'ship.png', 1 - math.random() * .3, math.random(5,20), math.random(1,3), math.random() * .2 + .05, math.random() * .5 + .2)
	ship.field_of_view = math.random(400,800)
	ship.player = player
	function ship:on_die()
		ENEMIES[self] = nil
		add_explosion(self:pos():unpack())

		local item = math.random(1,9)
		if item <= 3 then
			drop(IMAGES['cash.png'], 'cash', self.shape:center())
		elseif item == 4 then
			drop(IMAGES['apple.png'], 'health', self.shape:center())
		elseif item == 5 then
			drop(IMAGES['ammo.png'], 'ammo', self.shape:center())
		elseif item == 6 then
			drop(IMAGES['scope.png'], 'scope', self.shape:center())
		elseif item == 7 then
			drop(IMAGES['speed.png'], 'speed', self.shape:center())
		end
	end

	ENEMIES[ship] = ship
end

Platform = class{name = 'Platform', function(self, x,y, player)
	self.shape = HC.addRectangle(x,y+34,87,78)
	self.shape.class = Platform
	self.shape.self = self
	self.pos = vector(x,y)

	self.life = 1
	self.endurance = .1
	Timer.add(0.2, function(f)
		if self.life < 0 then return end
		local p = vector(self.shape:center())
		spawnShip(self.pos + vector(64,0):rotate_inplace(math.random() * 2 * math.pi), player)
		Timer.add(math.random(30,60), f)
	end)

	self.smoke = {}
	Timer.add(.2, function(f)
		if self.life < 0 then return end
		local cloud = {
			pos = vector(x + 23, y+17),
			dir = vector(0,-math.random(20,40)):rotate_inplace( (math.random() * 2 - 1) * math.pi/8 ),
			r = 0, rt = math.random() * math.pi,
			t = 0, maxt = math.random() + 5,
			s = math.random() * .3 + .85
		}
		self.smoke[cloud] = cloud
		Timer.add(math.random() * .2 + .4, f)
	end)

	ENTITIES[self] = self
	ENEMIES[self] = self
end}

function Platform:draw()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(IMAGES['platform.png'], self.pos:unpack())

	for _,cloud in pairs(self.smoke) do
		love.graphics.draw(IMAGES['smoke.png'], cloud.pos.x, cloud.pos.y, cloud.r, cloud.s, cloud.s, 16,16)
	end

	love.graphics.setColor(200,30,20,128)
	love.graphics.rectangle('fill', self.pos.x, self.pos.y - 10, 100, 5)
	love.graphics.setColor(30,200,30)
	love.graphics.rectangle('fill', self.pos.x, self.pos.y - 10, self.life*100, 5)
end

function Platform:update(dt)
	local to_remove = {}
	for _,cloud in pairs(self.smoke) do
		cloud.t = cloud.t + dt
		if cloud.t > cloud.maxt then
			to_remove[#to_remove+1] = cloud
		end
		cloud.pos = cloud.pos + cloud.dir * dt
		cloud.r = cloud.r + cloud.rt * dt
	end
	for _,c in ipairs(to_remove) do self.smoke[c] = nil end
end

function Platform:destroy()
	audio.play(SOUNDS['explode2.ogg'])
	HC.remove(self.shape)
	ENTITIES[self] = nil
	ENEMIES[self] = nil
	love.graphics.setColor(255,255,255)
	love.graphics.setColor(255,255,255)
end

function Platform:on_die()
	add_explosion( self.pos:unpack() )
	drop(IMAGES['cash.png'], 'cash', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['cash.png'], 'cash', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['cash.png'], 'cash', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['cash.png'], 'cash', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['cash.png'], 'cash', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['apple.png'], 'health', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['ammo.png'], 'ammo', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['scope.png'], 'scope', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
	drop(IMAGES['speed.png'], 'speed', (vector(self.shape:center())
		+ vector((math.random()*2-1) * 32, (math.random()*2-1)*32)):unpack())
end

Platform.collision_handler = {}
Platform.collision_handler[Ship] = function(self, ship, dx,dy)
	ship.shape:move(-dx,-dy)
end

Platform.collision_handler[Bullet] = function(self, bullet)
	self.life = self.life - math.max(bullet.damage * self.endurance, 0)
	if self.life < 0 then
		self:on_die()
		self:destroy()
	end

	bullet:destroy()
end

package.loaded[...] = Player
