GS     = require 'hump.gamestate'
HC     = require 'hardoncollider'
class  = require 'hardoncollider.class'
vector = require 'hardoncollider.vector'
Timer  = require 'hump.timer'
camera = require 'hump.camera'
require 'strict'
require 'AnAL'
no_game_code = nil

-- font cache
FONTS = setmetatable({}, {
	__index = function(t,size)
		t[size] = love.graphics.newFont(size)
		return t[size]
	end})

-- image cache
IMAGES = setmetatable({}, {
	__index = function(t, fn)
		local img = love.graphics.newImage('img/'..fn)
		img:setWrap('repeat', 'repeat')
		t[fn] = img
		return img
	end})

-- sound cache
SOUNDS = setmetatable({}, {
	__index = function(t, fn)
		t[fn] = love.sound.newSoundData('sound/'..fn)
		return t[fn]
	end})

SCR_WIDTH = 0
SCR_HEIGHT = 0

audio = { sources = {} }
function audio.play(sound, volume, loop)
	local source = love.audio.newSource(sound)
	source:setVolume(volume or 1)
	source:setLooping(loop)
	if not loop then
		local len = sound:getSize() / (sound:getBits() / 8 * sound:getChannels() * sound:getSampleRate())
		Timer.add(len + 0.1, function() audio.stop(source) end)
	end
	source:play()
	audio.sources[source] = source
	return source
end

function audio.stream(file, volume)
	local source = love.audio.newSource('sound/'..file, 'stream')
	source:setVolume(volume or 1)
	source:setLooping(true)
	source:play()
	audio.sources[source] = source
	return source
end

function audio.stopAll()
	for _, s in pairs(audio.sources) do
		s:stop()
	end
	audio.sources = {}
	audio.streaming = {}
end

function audio.stop(s)
	s:stop()
	audio.sources[s] = nil
end

states = {}
states.menu  = require 'states.menu'
states.intro = require 'states.intro'
states.game  = require 'states.game'
states.outro = require 'states.end'

function love.load()
	math.randomseed(os.time())

	SCR_WIDTH = love.graphics.getWidth()
	SCR_HEIGHT = love.graphics.getHeight()

	GS.registerEvents()
	GS.switch(states.menu)
end

function love.update(dt)
	Timer.update(dt)
end
