local state = GS.new()
package.loaded[...] = state

-- ----------------------- --
--     STATE FUNCTIONS     --
-- ----------------------- --
function state:init()
	self.quad_bg = love.graphics.newQuad(0,0, SCR_WIDTH * 2, SCR_HEIGHT * 2, 512,512)
end

function state:enter()
	self.bg_pos = 0
	self.update_bg_pos = Timer.Oscillator(40, function(f)
		self.bg_pos = -f * SCR_HEIGHT
	end)

	self.scroll_pos = SCR_HEIGHT
	self.update_scroll_pos = function() end
	Timer.add(3, function()
		self.update_scroll_pos = Timer.Interpolator(50, function(f)
			self.scroll_pos = (1 - 2.1*f) * SCR_HEIGHT
		end)

		Timer.add(40, function()
			self.fadeIn = Timer.Interpolator(3, function(f)
				self.press_start_alpha = f * 255
			end)
		end)
	end)

	self.black_alpha = 0
	self.fadeIn = Timer.Interpolator(3, function(f)
		self.black_alpha = (1 - f) * 255
	end)

	self.press_start_alpha = 0
end

function state:leave()
	audio.stopAll()
end

local story = [[It is the year 130 AF. Barbarism replaced a once peaceful, cultural civilisation. Clans and gangs roam the waters, pillaging every every defenseless settlement they encounter. There have been few attempts to restore order in this hostile and chaotic world. None achieved it's goal.

You were born and raised in a small settlement located in a region that was previously known as the nation of Egypt. You and your family led a simple and relatively simple life. Until THEY came.

They have been terrorizing your community for two years. You are fed up and you can't take it anymore. You are determined to end your life in fear. You strive to be a new beacon of hope.

It's time you took your gun and put end to this.]]

function state:draw()
	love.graphics.setColor(255,255,255)
	love.graphics.drawq(IMAGES['water02.jpg'], self.quad_bg, 0, self.bg_pos)

	love.graphics.setFont(FONTS[30])
	love.graphics.printf(story, 25, self.scroll_pos, SCR_WIDTH - 50, 'center')

	love.graphics.setColor(255,255,255, self.press_start_alpha)
	local y = (SCR_HEIGHT - FONTS[30]:getHeight('PRESS START')) / 2
	love.graphics.printf('PRESS START', 0, y, SCR_WIDTH, 'center')

	love.graphics.setColor(0,0,0, self.black_alpha)
	love.graphics.rectangle('fill', 0,0, SCR_WIDTH, SCR_HEIGHT)
end

function state:update(dt)
	self.fadeIn(dt)
	self.update_bg_pos(dt)
	self.update_scroll_pos(dt)
end

function state:keyreleased(key)
	if key == 'return' then
		self.fadeIn = Timer.Interpolator(1, function(f)
			self.black_alpha = f * 255
		end)
		Timer.add(1, function() GS.switch(states.game) end)
	end
end

return state
